//
// Copyright (c) 2002 - Novell Inc. - All Rights Reserved.
//
// File:         network.js
// Description:  Javascript networking utilities
//
// This software is the proprietary information of Novell
// Inc.
//


var hex = ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
   "A", "B", "C", "D", "E", "F"];

//
// Checks if the string specified is a valid IP. Valid IP
// addresses consist of 4 byte values separated by
// periods. Each byte value corresponds to a decimal 
// number between 0 and 255. The byte cannot begin with a 
// 0 unless its value is 0.
//
function isValidIP(s) {
   var re = /^(0|[1-9]\d{0,2})\.(0|[1-9]\d{0,2})\.(0|[1-9]\d{0,2})\.(0|[1-9]\d{0,2})$/

   var matchArray = re.exec(s);
   if (!matchArray)
      return false;

   for (var i=1; i<5; i++)
      if (matchArray[i] > 255)
         return false;

   return true;
}

//
// Checks if the string specified is a valid multicast address
// Valid multicast addresses can only have values between 224 and 239
// for the fist subnet.
//
function isValidMulticastAddress(s){
    if (isValidIP(s)){
        var subNetlist = s.split( "." );
        var firstSubNet = parseInt(subNetlist[0]);
        if ((firstSubNet >= 224) && (firstSubNet <= 239)) return true;
    }
    return false;
}	


//
// Checks if the string specified follows the rules for
// ARPANET host names. Valid host names must start with a
// letter, end with a letter or digit, and have as
// interior letters, digits, and hyphen. They also must
// be 63 characters or less.
//
function isValidHostName(s) {
   var re = /^[a-zA-Z][a-zA-Z0-9\-]*[a-zA-Z0-9]$/
   return (s.length <= 63 && re.test(s));
}

//
// Checks if the string specified is a valid domain name.
// Valid domain names consist of one or more host names
// separated by periods.
// 
function isValidDNS(s) {
   var labels = s.split(".");
   for (var i=0; i < labels.length; i++)
      if (!isValidHostName(labels[i]))
         return false;

   return true;
}

//
// Checks if the string specified is a valid scheme name.
// Valid domain names consist of one or more host names
// separated by periods.
// 
function isValidScheme(s) {
   if ("http" == s ||
		"ftp" == s ) {
	   return true;
   }
   return false;
}

//
// Checks if the string specified is a valid path.
// Valid domain names consist of one or more host names
// separated by periods.
// 
function isValidPath(s) {
   if (-1 == s.indexOf(" ")) {
	   return true;
   }
   return false;
}

//
// Checks if the string specified is a valid path.
// Valid domain names consist of one or more host names
// separated by periods.
// 
function isValidFileName(s) {
   if (-1 == s.indexOf(" ")) {
	   return true;
   }
   return false;
}

//
// Checks if the string specified is a valid path.
// Valid domain names consist of one or more host names
// separated by periods.
// 
function isValidFileExtension(s) {
   if (-1 == s.indexOf(" ")) {
	   return true;
   }
   return false;
}

//
// Checks if the string specified is a valid port.
// Valid port names must correspond to a decimal value
// between 1 and 65535. Also, port numbers cannot start
// with the digit 0.
//
function isValidPort(p) {
   var re = /^[1-9][0-9]*$/
   return (re.exec(p) != null && p <= 65535);
}

//
// Checks for valid IP range values.
// 
function isValidPortRange(p) {
	var iChar = p.indexOf("-");
	var firstPort = p.substring(0,iChar);
	var secondPort = p.substring(iChar+1);
	if (-1 == iChar ||
		!isValidPort( firstPort ) ||
		!isValidPort( secondPort ) ||
		1*firstPort > 1*secondPort ) {
		return false;
	}
	return true;
}

//
// Checks if the given string is a valid IP address or 
// a valid NDS name.
//
function isValidIPOrDNS( s ){
   if (! isValidIP(s) && ! isValidDNS(s)) {
      return false;
   }
   return true;
}  

//
// Converts a decimal number to the equivalent, binary,
// ternary, quintal, octal, or hexadecimal value depending
// on the radix specified. For example, radix 2 (binary)
// converts 245 to 11110101.
//
function convertDecimal(n, radix) {
   var s = "";
   while (n >= radix) {
      s += hex[n % radix];
      n = Math.floor(n / radix);
   }
   return transpose(s += hex[n]);
}

//
// Reverses the order of the characters in a string. For
// example, "10101111" gets converted to "11110101".
//
function transpose(s) {
   var t = "";
   for (var i=s.length-1; i >=0; i--)
      t += s.charAt(i);
   return t;
}

//
// Checks if the string specified is a valid subnet
// mask.
//
function isValidSubnetMask(s) {

   if (!isValidIP(s))
      return false;

   var mbin = "";
   var bytes = s.split(".");
   for (var i=0; i<4; i++) {
      mbin += convertDecimal(bytes[i], 2);
   }

   var pos0 = mbin.indexOf("0");
   var pos1 = mbin.indexOf("1", pos0+1);
   return (pos1 == -1);
}

//
// Computes the subnet from the subnet mask and the
// IP address.
//
function computeSubnet(mask, addr) {
   var s = "";
   var first = true;
   var mbytes = mask.split(".");
   var abytes = addr.split(".");
   for (var i=0; i < 4; i++) {
      if (first) first = false; else s += ".";
      s += (mbytes[i] & abytes[i]);
   }
   return s;
}

//
// Checks if an IP address belongs to the specified
// subnet. Subnet is identified by the subnet IP and
// mask.
// 
function isValidSubnetAddress(subnet, mask, addr) {

   if (!isValidIP(addr))
      return false;

   if (subnet.length > 0) {
      var s = computeSubnet(mask, addr);
      if (s != subnet) return false;
   }

   return true;
}

//
// Checks for valid IP range values.
// 
function isValidIPRange(p) {
	var iChar = p.indexOf("-");
	var firstIP = p.substring(0,iChar);
	var secondIP = p.substring(iChar+1);
	if (-1 == iChar ||
		!isValidIP( firstIP ) ||
		!isValidIP( secondIP ) ) {
		return false;
	}
	while( 0 < firstIP.length )	{
		iChar = firstIP.indexOf(".");
		if ( -1 == iChar ) {
			if ( 1*firstIP > 1*secondIP ) {
				return false;
			}
			return true;
		}
		else if ( 1*(firstIP.substring(0,iChar)) < 1*(secondIP.substring(0,secondIP.indexOf("."))) ) {
			return true;
		}
		else if ( 1*(firstIP.substring(0,iChar)) > 1*(secondIP.substring(0,secondIP.indexOf("."))) ) {
			return false;
		}
		firstIP = firstIP.substring(1+iChar);
		secondIP = secondIP.substring(1+secondIP.indexOf("."));
	}
}

//
// Checks if a mask is valid for CIDR notation.
// 
function isValidCIDRValue(p) {
	var iSlash = p.indexOf("/");
	if (-1 == iSlash ||
		!isValidIP( p.substring(0,iSlash) ) ||
		!isValidCIDRSubnetMask( p.substring(iSlash+1) ) ) {
		return false;
	}
	return true;
}

//
// Checks if a mask is valid for CIDR notation.
// 
function isValidCIDRSubnetMask(mask) {
   var re = /^[1-9][0-9]*$/
   return (re.exec(mask) != null && mask <= 32);
}